#include "conf.h"
#include "sysdep.h"

#include "screen.h"
#include "structs.h"
#include "utils.h"
#include "db.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "spells.h"

/* Local functions */
void start_auction(struct char_data * ch, struct obj_data * obj, int bid);
void auc_stat(struct char_data * ch, struct obj_data *obj);
void stop_auction(int type, struct char_data * ch);
void check_auction(void);
void auc_send_to_all(int state, bool buyer);
char buf[MAX_STRING_LENGTH];
ACMD(do_auction);
ACMD(do_bid);

int curbid = 0;				/* current bid on item being auctioned */
int aucstat = AUC_NULL_STATE;		/* state of auction.. first_bid etc.. */
struct obj_data *obj_selling = NULL;	/* current object for sale */
struct char_data *ch_selling = NULL;	/* current character selling obj */
struct char_data *ch_buying  = NULL;	/* current character buying the object */

char *auctioneer[AUC_BID + 1] = {
	
	"The auctioneer auctions, '$n puts $p up for sale at %d coins.'",
	"The auctioneer auctions, '$p at %d coins going once!.'",
	"The auctioneer auctions, '$p at %d coins going twice!.'",
	"The auctioneer auctions, 'Last call: $p going to $n for %d coins.'",
	"The auctioneer auctions, 'Unfortunately $p is unsold, returning it to $n.'",
	"The auctioneer auctions, 'SOLD! $p to $n for %d coins!.'",
	"The auctioneer auctions, 'Sorry, $n has cancelled the auction.'",
	"The auctioneer auctions, 'Sorry, $n has left us, the auction can't go on.'",
	"The auctioneer auctions, 'Sorry, $p has been confiscated, shame on you $n.'",
	"The auctioneer tells you, '$n is selling $p for %d gold.'",
	"The auctioneer auctions, '$n bids %d coins on $p.'"
};

void start_auction(struct char_data * ch, struct obj_data * obj, int bid)
{
  /* Take object from character and set variables */
  obj_from_char(obj);
  obj_selling = obj;
  ch_selling	= ch;
  ch_buying = NULL;
  curbid = bid;

  /* Tell th character where his item went */
  send_to_char(ch_selling, "%s magicly flies away from your hands to be auctioned!\r\n", 
	  CAP(obj_selling->short_description));
	
  /* Anounce the item is being sold */
  auc_send_to_all(AUC_NULL_STATE, FALSE);

  aucstat = AUC_OFFERING;
}

void check_auction(void)
{
  switch (aucstat) {
  case AUC_NULL_STATE:
	return;
  case AUC_OFFERING:
	auc_send_to_all(AUC_OFFERING, FALSE);
	aucstat = AUC_GOING_ONCE;
	return;
  case AUC_GOING_ONCE:
	auc_send_to_all(AUC_GOING_ONCE, FALSE);
	aucstat = AUC_GOING_TWICE;
    return;
  case AUC_GOING_TWICE:
	auc_send_to_all(AUC_GOING_TWICE, FALSE);
	aucstat = AUC_LAST_CALL;
	return;
  case AUC_LAST_CALL:
	if (ch_buying == NULL) {
	  auc_send_to_all(AUC_LAST_CALL, FALSE);
			
      send_to_char(ch_selling, "%s flies out the sky and into your hands.\r\n", CAP(obj_selling->short_description));
	  obj_to_char(obj_selling, ch_selling);
			
	  /* Reset auctioning values */
	  obj_selling = NULL;
	  ch_selling = NULL;
	  ch_buying = NULL;
	  curbid = 0;
	  aucstat = AUC_NULL_STATE;
	  return;
	} else {
	  auc_send_to_all(AUC_SOLD, TRUE);
					
	  /* Give the object to the buyer */
	  obj_to_char(obj_selling, ch_buying);
	  send_to_char(ch_buying, "%s flies out the sky and into your hands, what a steel!\r\n", CAP(obj_selling->short_description));
		
	  send_to_char(ch_selling, "Congrats! You have sold %s for %d coins!\r\n", obj_selling->short_description, curbid);
	
	  /* Give selling char the money for his stuff */
	  GET_GOLD(ch_selling) += curbid;
			
	  /* Reset auctioning values */
	  obj_selling = NULL;
	  ch_selling = NULL;
	  ch_buying = NULL;
	  curbid = 0;
	  aucstat = AUC_NULL_STATE;
	  return;						
	}
  }
}


ACMD(do_auction)
{
  char arg1[MAX_INPUT_LENGTH];
  char arg2[MAX_INPUT_LENGTH];
  struct obj_data *obj;
  int bid = 0;
	
  if(IS_NPC(ch))
	return;

  two_arguments(argument, arg1, arg2);
	
  if (!*arg1) {
	send_to_char(ch, "Auction what?\r\n");
	return;
  }	else if (is_abbrev(arg1, "cancel") || is_abbrev(arg1, "stop")) {
	if ((ch != ch_selling && GET_LEVEL(ch) < LVL_GRGOD) || aucstat == AUC_NULL_STATE) {
	  send_to_char(ch, "You're not even selling anything!\r\n");
	  return;
	} else if (ch == ch_selling) {
	  stop_auction(AUC_NORMAL_CANCEL, NULL);
	  return;
	} else {
	  stop_auction(AUC_WIZ_CANCEL, ch);
	}
  }	else if (is_abbrev(arg1, "stats") || is_abbrev(arg1, "identify")) {
	auc_stat(ch, obj_selling);
	return;
  }	else if (!(obj = get_obj_in_list_vis(ch, arg1, NULL, ch->carrying))) {
	send_to_char(ch, "You don't seem to have %s %s.\r\n", AN(arg1), arg1);
	return;
  } else if (!*arg2 && (bid = obj->obj_flags.cost) <= 0) {
	send_to_char(ch, "What should be the minimum bid?\r\n");
	return;
  }	else if (*arg2 && (bid = atoi(arg2)) <= 0) {
	send_to_char(ch, "Come on? One coin at least?\r\n");
	return;
  } else if (aucstat != AUC_NULL_STATE) {
	send_to_char(ch, "Sorry, but %s is already auctioning %s at %d coins!\r\n", GET_NAME(ch_selling),
				obj_selling->short_description, curbid);
	return;
  }	else if (OBJ_FLAGGED(obj, ITEM_NOSELL))	{
	send_to_char(ch, "Sorry but you can't sell that!\r\n");
	return;
  }	else {
	send_to_char(ch, "%s", CONFIG_OK);
	start_auction(ch, obj, bid);
	return;
  }
}


ACMD(do_bid)
{
  char arg[MAX_INPUT_LENGTH];
  int bid;

  if(IS_NPC(ch))
	return;

  one_argument(argument, arg);

  if (!*arg) {
	send_to_char(ch, "Bid yes, good idea, but HOW MUCH??\r\n");
	return;
  }	else if (aucstat == AUC_NULL_STATE){
	send_to_char(ch, "Thats very enthusiastic of you, but nothing is being SOLD!\r\n");
	return;
  }	else if (ch == ch_selling) {
	send_to_char(ch, "Why bid on something your selling?  You can 'cancel' the auction!\r\n");
	return;
  }	else if ((bid = atoi(arg)) < ((int) curbid * 1.1 - 1) && ch_buying != NULL) {
	send_to_char(ch, "You must bid at least 10 percent more than the current bid. (%d)\r\n", (int) (curbid * 1.1));
	return;
  }	else if (ch_buying == NULL && bid < curbid) {
	send_to_char(ch, "You must at least bid the minimum!\r\n");
	return;
  }	else if (bid > GET_GOLD(ch) && GET_LEVEL(ch) < LVL_IMMORT) {
	send_to_char(ch, "You don't have that much gold!\r\n");
	return;
  }	else {
	if (ch == ch_buying && GET_LEVEL(ch) < LVL_IMMORT)
      GET_GOLD(ch) -= (bid - curbid);
	else  {
	  if (GET_LEVEL(ch) < LVL_IMMORT)
		GET_GOLD(ch) -= bid;
			
	  if(!(ch_buying == NULL))
		GET_GOLD(ch_buying) += curbid;
	}
	curbid = bid;
	ch_buying = ch;
	
	auc_send_to_all(AUC_BID, TRUE);
		
	aucstat = AUC_OFFERING;
	return;
  }
}

void stop_auction(int type, struct char_data * ch)
{
  switch (type) {
	case AUC_NORMAL_CANCEL:
	  auc_send_to_all(AUC_NORMAL_CANCEL, FALSE);
      break;
	case AUC_QUIT_CANCEL:
      auc_send_to_all(AUC_QUIT_CANCEL, FALSE);
	  break;
	case AUC_WIZ_CANCEL:
      auc_send_to_all(AUC_WIZ_CANCEL, FALSE);
	  break;
	default:
	  send_to_char(ch, "Sorry, that is an unrecognised cancel command, please report.");
	  return;
  }

  if (type != AUC_WIZ_CANCEL) {
	send_to_char(ch_selling, "%s flies out the sky and into your hands.\r\n", CAP(obj_selling->short_description));
	obj_to_char(obj_selling, ch_selling);
  }	else {
	send_to_char(ch_selling, "%s flies out the sky and into your hands.\r\n", CAP(obj_selling->short_description));
	obj_to_char(obj_selling, ch);
  }
	
  if (!(ch_buying == NULL))
	GET_GOLD(ch_buying) += curbid;
		
  obj_selling = NULL;
  ch_selling = NULL;
  ch_buying = NULL;
  curbid = 0;

  aucstat = AUC_NULL_STATE;
}

void auc_stat(struct char_data *ch, struct obj_data *obj)
{
  char buf[MAX_STRING_LENGTH];
	
	if (aucstat == AUC_NULL_STATE) {
	send_to_char(ch, "Nothing is being auctioned!\r\n");
	return;
  }	else if (ch == ch_selling) {
	send_to_char(ch, "You should have found that out BEFORE auctioning it!\r\n");
	return;
  }	else if (GET_GOLD(ch) < 500) {
	send_to_char(ch, "You can't afford to find the stats on that, it costs 500 coins!\r\n");
	return;
  }	else {
	/* auctioneer tells the character the auction details */
	sprintf(buf, auctioneer[AUC_STAT], curbid);
	send_to_char(ch, "%s", CCRED(ch, C_NRM));
    act(buf, TRUE, ch_selling, obj, ch, TO_VICT | TO_SLEEP);
	send_to_char(ch, "%s", CCNRM(ch, C_NRM));
	GET_GOLD(ch) -= 500;	

	call_magic(ch, NULL, obj_selling, SPELL_IDENTIFY, 30, CAST_SPELL);
  }
}

void auc_send_to_all(int state, bool buyer)
{
  struct descriptor_data *i;
  char buf[MAX_STRING_LENGTH];

  for (i = descriptor_list; i; i = i->next) {
	if (STATE(i) != CON_PLAYING)
      continue;
	
	if (PRF_FLAGGED(i->character, PRF_NOAUCT))
	  continue;
	
	send_to_char(i->character, "%s", CCMAG(i->character, C_NRM));
	sprintf(buf, auctioneer[state], curbid);
	
	if (buyer)
	  act(buf, TRUE, ch_buying, obj_selling, i->character, TO_VICT | TO_SLEEP);
	else
	  act(buf, TRUE, ch_selling, obj_selling, i->character, TO_VICT | TO_SLEEP);
	
	send_to_char(i->character, "%s", CCNRM(i->character, C_NRM));
  }
}

